/**
 * @module
 * @public
 * @description
 * Description of the change_detection module
 */
library angular2.change_detection;

export "src/change_detection/parser/ast.dart"
    show
        ASTWithSource,
        AST,
        AstTransformer,
        AccessMember,
        LiteralArray,
        ImplicitReceiver;
export "src/change_detection/parser/lexer.dart" show Lexer;
export "src/change_detection/parser/parser.dart" show Parser;
export "src/change_detection/parser/locals.dart" show Locals;
export "src/change_detection/exceptions.dart"
    show ExpressionChangedAfterItHasBeenChecked, ChangeDetectionError;
export "src/change_detection/interfaces.dart"
    show ProtoChangeDetector, ChangeDispatcher, ChangeDetector, ChangeDetection;
export "src/change_detection/constants.dart"
    show CHECK_ONCE, CHECK_ALWAYS, DETACHED, CHECKED, ON_PUSH, DEFAULT;
export "src/change_detection/proto_change_detector.dart"
    show DynamicProtoChangeDetector, JitProtoChangeDetector;
export "src/change_detection/binding_record.dart" show BindingRecord;
export "src/change_detection/directive_record.dart" show DirectiveRecord;
export "src/change_detection/dynamic_change_detector.dart"
    show DynamicChangeDetector;
export "src/change_detection/binding_propagation_config.dart"
    show BindingPropagationConfig;
export "src/change_detection/pipes/pipe_registry.dart";
export "src/change_detection/change_detection_util.dart" show uninitialized;
export "src/change_detection/pipes/pipe.dart";
import "src/change_detection/proto_change_detector.dart"
    show DynamicProtoChangeDetector, JitProtoChangeDetector;
import "src/change_detection/pipes/pipe_registry.dart" show PipeRegistry;
import "src/change_detection/pipes/iterable_changes.dart"
    show IterableChangesFactory;
import "src/change_detection/pipes/keyvalue_changes.dart"
    show KeyValueChangesFactory;
import "src/change_detection/pipes/null_pipe.dart" show NullPipeFactory;
import "src/change_detection/constants.dart" show DEFAULT;
import "src/change_detection/interfaces.dart"
    show ChangeDetection, ProtoChangeDetector;

var defaultPipes = {
  "iterableDiff": [new IterableChangesFactory(), new NullPipeFactory()],
  "keyValDiff": [new KeyValueChangesFactory(), new NullPipeFactory()]
}; /**
 * @exportedAs angular2/change_detection
 */
class DynamicChangeDetection extends ChangeDetection {
  PipeRegistry registry;
  DynamicChangeDetection(PipeRegistry registry) : super() {
    /* super call moved to initializer */;
    this.registry = registry;
  }
  ProtoChangeDetector createProtoChangeDetector(String name,
      [String changeControlStrategy = DEFAULT]) {
    return new DynamicProtoChangeDetector(this.registry, changeControlStrategy);
  }
} /**
 * @exportedAs angular2/change_detection
 */
class JitChangeDetection extends ChangeDetection {
  PipeRegistry registry;
  JitChangeDetection(PipeRegistry registry) : super() {
    /* super call moved to initializer */;
    this.registry = registry;
  }
  ProtoChangeDetector createProtoChangeDetector(String name,
      [String changeControlStrategy = DEFAULT]) {
    return new JitProtoChangeDetector(this.registry, changeControlStrategy);
  }
}
var _registry = new PipeRegistry(defaultPipes);
var dynamicChangeDetection = new DynamicChangeDetection(_registry);
var jitChangeDetection = new JitChangeDetection(_registry);
