library angular2.src.core.annotations.di;

import "package:angular2/src/facade/lang.dart" show CONST;
import "package:angular2/di.dart"
    show
        DependencyAnnotation; /**
 * The directive can inject an emitter function that would emit events onto the
 * directive host element.
 */

class EventEmitter extends DependencyAnnotation {
  final String eventName;
  @CONST() const EventEmitter(eventName)
      : eventName = eventName,
        super();
  get token {
    return Function;
  }
} /**
 * The directive can inject a property setter that would allow setting this property on the
 * host element
 */
class PropertySetter extends DependencyAnnotation {
  final String propName;
  @CONST() const PropertySetter(propName)
      : propName = propName,
        super();
  get token {
    return Function;
  }
} /**
 * The directive can inject the value of an attribute of the host element
 */
class Attribute extends DependencyAnnotation {
  final String attributeName;
  @CONST() const Attribute(attributeName)
      : attributeName = attributeName,
        super();
  get token {
    //Normally one would default a token to a type of an injected value but here
    //the type of a variable is "string" and we can't use primitive type as a return value
    //so we use instance of Attribute instead. This doesn't matter much in practice as arguments
    //with @Attribute annotation are injected by ElementInjector that doesn't take tokens into account.
    return this;
  }
} /**
 * The directive can inject an query that would reflect a list of ancestor directives
 */
class Query extends DependencyAnnotation {
  final directive;
  @CONST() const Query(directive)
      : directive = directive,
        super();
}
