library angular2.src.core.compiler.view_container;

import "package:angular2/src/facade/collection.dart"
    show ListWrapper, MapWrapper, List;
import "package:angular2/src/facade/lang.dart" show BaseException;
import "package:angular2/di.dart" show Injector;
import "package:angular2/src/core/compiler/element_injector.dart" as eiModule;
import "package:angular2/src/facade/lang.dart" show isPresent, isBlank;
import "package:angular2/src/render/api.dart" as renderApi;
import "view.dart" as viewModule;
import "view_factory.dart" as vfModule; /**
 * @exportedAs angular2/template
 */

class ViewContainer {
  renderApi.ViewContainerRef render;
  vfModule.ViewFactory viewFactory;
  viewModule.AppView parentView;
  viewModule.AppProtoView defaultProtoView;
  List<viewModule.AppView> _views;
  eiModule.ElementInjector elementInjector;
  Injector appInjector;
  eiModule.ElementInjector hostElementInjector;
  ViewContainer(vfModule.ViewFactory viewFactory, viewModule.AppView parentView,
      viewModule.AppProtoView defaultProtoView,
      eiModule.ElementInjector elementInjector) {
    this.viewFactory = viewFactory;
    this.render = null;
    this.parentView = parentView;
    this.defaultProtoView = defaultProtoView;
    this.elementInjector =
        elementInjector; // The order in this list matches the DOM order.
    this._views = [];
    this.appInjector = null;
    this.hostElementInjector = null;
  }
  internalHydrateRecurse(renderApi.ViewContainerRef render,
      Injector appInjector, eiModule.ElementInjector hostElementInjector) {
    this.render = render;
    this.appInjector = appInjector;
    this.hostElementInjector = hostElementInjector;
  }
  internalDehydrateRecurse() {
    this.appInjector = null;
    this.hostElementInjector = null;
    this.render = null; // Note: We don't call clear here,
    // as we don't want to change the render side
    // (i.e. don't deattach views on the render side),
    // as the render side does its own recursion.
    for (var i = this._views.length - 1; i >= 0; i--) {
      var view = this._views[i];
      view.changeDetector.remove();
      this._unlinkElementInjectors(view);
      view.internalDehydrateRecurse();
      this.viewFactory.returnView(view);
    }
    this._views = [];
  }
  clear() {
    for (var i = this._views.length - 1; i >= 0; i--) {
      this.remove(i);
    }
  }
  viewModule.AppView get(num index) {
    return this._views[index];
  }
  get length {
    return this._views.length;
  }
  _siblingInjectorToLinkAfter(num index) {
    if (index == 0) return null;
    return ListWrapper.last(this._views[index - 1].rootElementInjectors);
  }
  hydrated() {
    return isPresent(this.appInjector);
  } // TODO(rado): profile and decide whether bounds checks should be added
  // to the methods below.
  viewModule.AppView create([atIndex = -1]) {
    if (!this.hydrated()) throw new BaseException(
        "Cannot create views on a dehydrated ViewContainer");
    var newView = this.viewFactory.getView(
        this.defaultProtoView); // insertion must come before hydration so that element injector trees are attached.
    this._insertWithoutRender(newView,
        atIndex); // hydration must come before changing the render side,
    // as it acquires the render views.
    newView.hydrate(this.appInjector, this.hostElementInjector,
        this.parentView.context, this.parentView.locals);
    this.defaultProtoView.renderer.insertViewIntoContainer(
        this.render, newView.render, atIndex);
    return newView;
  }
  viewModule.AppView insert(view, [atIndex = -1]) {
    this._insertWithoutRender(view, atIndex);
    this.defaultProtoView.renderer.insertViewIntoContainer(
        this.render, view.render, atIndex);
    return view;
  }
  viewModule.AppView _insertWithoutRender(view, [atIndex = -1]) {
    if (atIndex == -1) atIndex = this._views.length;
    ListWrapper.insert(this._views, atIndex, view);
    this.parentView.changeDetector.addChild(view.changeDetector);
    this._linkElementInjectors(this._siblingInjectorToLinkAfter(atIndex), view);
    return view;
  }
  remove([atIndex = -1]) {
    if (atIndex == -1) atIndex = this._views.length - 1;
    var view = this.detach(atIndex);
    view.dehydrate();
    this.viewFactory.returnView(view);
  } /**
   * The method can be used together with insert to implement a view move, i.e.
   * moving the dom nodes while the directives in the view stay intact.
   */
  viewModule.AppView detach([atIndex = -1]) {
    if (atIndex == -1) atIndex = this._views.length - 1;
    var detachedView = this.get(atIndex);
    ListWrapper.removeAt(this._views, atIndex);
    this.defaultProtoView.renderer.detachViewFromContainer(
        this.render, atIndex);
    detachedView.changeDetector.remove();
    this._unlinkElementInjectors(detachedView);
    return detachedView;
  }
  contentTagContainers() {
    return this._views;
  }
  _linkElementInjectors(sibling, view) {
    for (var i = view.rootElementInjectors.length - 1; i >= 0; i--) {
      view.rootElementInjectors[i].linkAfter(this.elementInjector, sibling);
    }
  }
  _unlinkElementInjectors(view) {
    for (var i = 0; i < view.rootElementInjectors.length; ++i) {
      view.rootElementInjectors[i].unlink();
    }
  }
}
