library angular2.src.di.exceptions;

import "package:angular2/src/facade/collection.dart" show ListWrapper, List;
import "package:angular2/src/facade/lang.dart" show stringify;

findFirstClosedCycle(List keys) {
  var res = [];
  for (var i = 0; i < keys.length; ++i) {
    if (ListWrapper.contains(res, keys[i])) {
      ListWrapper.push(res, keys[i]);
      return res;
    } else {
      ListWrapper.push(res, keys[i]);
    }
  }
  return res;
}
constructResolvingPath(List keys) {
  if (keys.length > 1) {
    var reversed = findFirstClosedCycle(ListWrapper.reversed(keys));
    var tokenStrs = ListWrapper.map(reversed, (k) => stringify(k.token));
    return " (" + tokenStrs.join(" -> ") + ")";
  } else {
    return "";
  }
}
class KeyMetadataError extends Error {}
class ProviderError extends Error {
  List keys;
  Function constructResolvingMessage;
  var message; // TODO(tbosch): Can't do key:Key as this results in a circular dependency!
  ProviderError(key, Function constructResolvingMessage) : super() {
    /* super call moved to initializer */;
    this.keys = [key];
    this.constructResolvingMessage = constructResolvingMessage;
    this.message = this.constructResolvingMessage(this.keys);
  } // TODO(tbosch): Can't do key:Key as this results in a circular dependency!
  addKey(key) {
    ListWrapper.push(this.keys, key);
    this.message = this.constructResolvingMessage(this.keys);
  }
  toString() {
    return this.message;
  }
}
class NoProviderError extends ProviderError {
  // TODO(tbosch): Can't do key:Key as this results in a circular dependency!
  NoProviderError(key) : super(key, (List keys) {
        var first = stringify(ListWrapper.first(keys).token);
        return '''No provider for ${ first}!${ constructResolvingPath ( keys )}''';
      }) {
    /* super call moved to initializer */;
  }
}
class AsyncBindingError extends ProviderError {
  // TODO(tbosch): Can't do key:Key as this results in a circular dependency!
  AsyncBindingError(key) : super(key, (List keys) {
        var first = stringify(ListWrapper.first(keys).token);
        return '''Cannot instantiate ${ first} synchronously. ''' +
            '''It is provided as a promise!${ constructResolvingPath ( keys )}''';
      }) {
    /* super call moved to initializer */;
  }
}
class CyclicDependencyError extends ProviderError {
  // TODO(tbosch): Can't do key:Key as this results in a circular dependency!
  CyclicDependencyError(key) : super(key, (List keys) {
        return '''Cannot instantiate cyclic dependency!${ constructResolvingPath ( keys )}''';
      }) {
    /* super call moved to initializer */;
  }
}
class InstantiationError extends ProviderError {
  // TODO(tbosch): Can't do key:Key as this results in a circular dependency!
  InstantiationError(originalException, key) : super(key, (List keys) {
        var first = stringify(ListWrapper.first(keys).token);
        return '''Error during instantiation of ${ first}!${ constructResolvingPath ( keys )}.''' +
            ''' ORIGINAL ERROR: ${ originalException}''';
      }) {
    /* super call moved to initializer */;
  }
}
class InvalidBindingError extends Error {
  String message;
  InvalidBindingError(binding) : super() {
    /* super call moved to initializer */;
    this.message = '''Invalid binding ${ binding}''';
  }
  toString() {
    return this.message;
  }
}
class NoAnnotationError extends Error {
  String message;
  NoAnnotationError(typeOrFunc) : super() {
    /* super call moved to initializer */;
    this.message =
        '''Cannot resolve all parameters for ${ stringify ( typeOrFunc )}''';
  }
  toString() {
    return this.message;
  }
}
