library angular2.src.render.api;

import "package:angular2/src/facade/lang.dart" show isPresent;
import "package:angular2/src/facade/async.dart" show Future;
import "package:angular2/src/facade/collection.dart" show List, Map;
import "package:angular2/change_detection.dart"
    show
        ASTWithSource; /**
 * General notes:
 * We are already parsing expressions on the render side:
 * - this makes the ElementBinders more compact
 *   (e.g. no need to distinguish interpolations from regular expressions from literals)
 * - allows to retrieve which properties should be accessed from the event
 *   by looking at the expression
 * - we need the parse at least for the `template` attribute to match
 *   directives in it
 * - render compiler is not on the critical path as
 *   its output will be stored in precompiled templates.
 */

class EventBinding {
  String fullName;
  ASTWithSource source;
  EventBinding(String fullName, ASTWithSource source) {
    this.fullName = fullName;
    this.source = source;
  }
}
class ElementBinder {
  num index;
  num parentIndex;
  num distanceToParent;
  List<DirectiveBinder> directives;
  ProtoViewDto nestedProtoView;
  Map<String, ASTWithSource> propertyBindings;
  Map<String, ASTWithSource> variableBindings; // Note: this contains a preprocessed AST
  // that replaced the values that should be extracted from the element
  // with a local name
  List<EventBinding> eventBindings;
  List<ASTWithSource> textBindings;
  Map<String, String> readAttributes;
  ElementBinder({index, parentIndex, distanceToParent, directives,
      nestedProtoView, propertyBindings, variableBindings, eventBindings,
      textBindings, readAttributes}) {
    this.index = index;
    this.parentIndex = parentIndex;
    this.distanceToParent = distanceToParent;
    this.directives = directives;
    this.nestedProtoView = nestedProtoView;
    this.propertyBindings = propertyBindings;
    this.variableBindings = variableBindings;
    this.eventBindings = eventBindings;
    this.textBindings = textBindings;
    this.readAttributes = readAttributes;
  }
}
class DirectiveBinder {
  // Index into the array of directives in the View instance
  dynamic directiveIndex;
  Map<String, ASTWithSource> propertyBindings; // Note: this contains a preprocessed AST
  // that replaced the values that should be extracted from the element
  // with a local name
  List<EventBinding> eventBindings;
  DirectiveBinder({directiveIndex, propertyBindings, eventBindings}) {
    this.directiveIndex = directiveIndex;
    this.propertyBindings = propertyBindings;
    this.eventBindings = eventBindings;
  }
}
class ProtoViewDto {
  ProtoViewRef render;
  List<ElementBinder> elementBinders;
  Map<String, String> variableBindings;
  ProtoViewDto({render, elementBinders, variableBindings}) {
    this.render = render;
    this.elementBinders = elementBinders;
    this.variableBindings = variableBindings;
  }
}
class DirectiveMetadata {
  static get DECORATOR_TYPE {
    return 0;
  }
  static get COMPONENT_TYPE {
    return 1;
  }
  static get VIEWPORT_TYPE {
    return 2;
  }
  dynamic id;
  String selector;
  bool compileChildren;
  Map<String, String> hostListeners;
  Map<String, String> properties;
  List<String> setters;
  List<String> readAttributes;
  num type;
  DirectiveMetadata({id, selector, compileChildren, hostListeners, properties,
      setters, readAttributes, type}) {
    this.id = id;
    this.selector = selector;
    this.compileChildren = isPresent(compileChildren) ? compileChildren : true;
    this.hostListeners = hostListeners;
    this.properties = properties;
    this.setters = setters;
    this.readAttributes = readAttributes;
    this.type = type;
  }
} // An opaque reference to a RenderProtoView
class ProtoViewRef {} // An opaque reference to a RenderView
class ViewRef {}
class ViewContainerRef {
  ViewRef view;
  num elementIndex;
  ViewContainerRef(ViewRef view, num elementIndex) {
    this.view = view;
    this.elementIndex = elementIndex;
  }
}
class ViewDefinition {
  String componentId;
  String absUrl;
  String template;
  List<DirectiveMetadata> directives;
  ViewDefinition({componentId, absUrl, template, directives}) {
    this.componentId = componentId;
    this.absUrl = absUrl;
    this.template = template;
    this.directives = directives;
  }
}
class Renderer {
  /**
   * Compiles a single RenderProtoView. Non recursive so that
   * we don't need to serialize all possible components over the wire,
   * but only the needed ones based on previous calls.
   */
  Future<ProtoViewDto> compile(ViewDefinition template) {
    return null;
  } /**
   * Sets the preset nested components,
   * which will be instantiated when this protoView is instantiated.
   * Note: We can't create new ProtoViewRefs here as we need to support cycles / recursive components.
   * @param {List<ProtoViewRef>} protoViewRefs
   *    RenderProtoView for every element with a component in this protoView or in a view container's protoView
   */
  mergeChildComponentProtoViews(
      ProtoViewRef protoViewRef, List<ProtoViewRef> componentProtoViewRefs) {
    return null;
  } /**
   * Creats a RenderProtoView that will create a root view for the given element,
   * i.e. it will not clone the element but only attach other proto views to it.
   * Contains a single nested component with the given componentId.
   */
  Future<ProtoViewDto> createRootProtoView(selectorOrElement, componentId) {
    return null;
  } /**
   * Creates a view and all of its nested child components.
   * @return {List<ViewRef>} depth first list of nested child components
   */
  List<ViewRef> createView(ProtoViewRef protoView) {
    return null;
  } /**
   * Destroys a view and returns it back into the pool.
   */
  void destroyView(ViewRef view) {
  } /**
   * Inserts a detached view into a viewContainer.
   */
  void insertViewIntoContainer(ViewContainerRef vcRef, ViewRef view, atIndex) {
  } /**
   * Detaches a view from a container so that it can be inserted later on
   * Note: We are not return the ViewRef as this can't be done in sync,
   * so we assume that the caller knows which view is in which spot...
   */
  void detachViewFromContainer(ViewContainerRef vcRef, num atIndex) {
  } /**
   * Sets a property on an element.
   * Note: This will fail if the property was not mentioned previously as a propertySetter
   * in the View.
   */
  void setElementProperty(ViewRef view, num elementIndex, String propertyName,
      dynamic propertyValue) {
  } /**
   * Installs a nested component in another view.
   * Note: only allowed if there is a dynamic component directive
   */
  void setDynamicComponentView(
      ViewRef view, num elementIndex, ViewRef nestedViewRef) {
  } /**
   * This will set the value for a text node.
   * Note: This needs to be separate from setElementProperty as we don't have ElementBinders
   * for text nodes in the RenderProtoView either.
   */
  void setText(ViewRef view, num textNodeIndex, String text) {
  } /**
   * Sets the dispatcher for all events that have been defined in the template or in directives
   * in the given view.
   */
  void setEventDispatcher(ViewRef viewRef, dynamic dispatcher) {
  } /**
   * To be called at the end of the VmTurn so the API can buffer calls
   */
  void flush() {}
} /**
 * A dispatcher for all events happening in a view.
 */
class EventDispatcher {
  /**
   * Called when an event was triggered for a on-* attribute on an element.
   * @param {Map<string, any>} locals Locals to be used to evaluate the
   *   event expressions
   */
  void dispatchEvent(
      num elementIndex, String eventName, Map<String, dynamic> locals) {}
}
