library angular2.src.render.dom.direct_dom_renderer;

import "package:angular2/di.dart" show Injectable;
import "package:angular2/src/facade/async.dart" show Future, PromiseWrapper;
import "package:angular2/src/facade/collection.dart" show List, ListWrapper;
import "package:angular2/src/facade/lang.dart" show isBlank, isPresent;
import "../api.dart" as api;
import "view/view.dart" show RenderView;
import "view/proto_view.dart" show RenderProtoView;
import "view/view_factory.dart" show ViewFactory;
import "compiler/compiler.dart" show Compiler;
import "shadow_dom/shadow_dom_strategy.dart" show ShadowDomStrategy;
import "view/proto_view_builder.dart" show ProtoViewBuilder;

_resolveViewContainer(api.ViewContainerRef vc) {
  return _resolveView(vc.view).viewContainers[vc.elementIndex];
}
_resolveView(DirectDomViewRef viewRef) {
  return isPresent(viewRef) ? viewRef.delegate : null;
}
_resolveProtoView(DirectDomProtoViewRef protoViewRef) {
  return isPresent(protoViewRef) ? protoViewRef.delegate : null;
}
_wrapView(RenderView view) {
  return new DirectDomViewRef(view);
}
_collectComponentChildViewRefs(view, [target = null]) {
  if (isBlank(target)) {
    target = [];
  }
  ListWrapper.push(target, _wrapView(view));
  ListWrapper.forEach(view.componentChildViews, (view) {
    if (isPresent(view)) {
      _collectComponentChildViewRefs(view, target);
    }
  });
  return target;
} // public so that the compiler can use it.
class DirectDomProtoViewRef extends api.ProtoViewRef {
  RenderProtoView delegate;
  DirectDomProtoViewRef(RenderProtoView delegate) : super() {
    /* super call moved to initializer */;
    this.delegate = delegate;
  }
}
class DirectDomViewRef extends api.ViewRef {
  RenderView delegate;
  DirectDomViewRef(RenderView delegate) : super() {
    /* super call moved to initializer */;
    this.delegate = delegate;
  }
}
@Injectable()
class DirectDomRenderer extends api.Renderer {
  Compiler _compiler;
  ViewFactory _viewFactory;
  ShadowDomStrategy _shadowDomStrategy;
  DirectDomRenderer(Compiler compiler, ViewFactory viewFactory,
      ShadowDomStrategy shadowDomStrategy)
      : super() {
    /* super call moved to initializer */;
    this._compiler = compiler;
    this._viewFactory = viewFactory;
    this._shadowDomStrategy = shadowDomStrategy;
  }
  Future<api.ProtoViewDto> compile(api.ViewDefinition template) {
    // Note: compiler already uses a DirectDomProtoViewRef, so we don't
    // need to do anything here
    return this._compiler.compile(template);
  }
  mergeChildComponentProtoViews(
      api.ProtoViewRef protoViewRef, List<api.ProtoViewRef> protoViewRefs) {
    _resolveProtoView(protoViewRef).mergeChildComponentProtoViews(
        ListWrapper.map(protoViewRefs, _resolveProtoView));
  }
  Future<api.ProtoViewDto> createRootProtoView(selectorOrElement, componentId) {
    var element = selectorOrElement;
    var rootProtoViewBuilder = new ProtoViewBuilder(element);
    rootProtoViewBuilder.setIsRootView(true);
    var elBinder = rootProtoViewBuilder.bindElement(element, "root element");
    elBinder.setComponentId(componentId);
    elBinder.bindDirective(0);
    this._shadowDomStrategy.processElement(null, componentId, element);
    return PromiseWrapper.resolve(rootProtoViewBuilder.build());
  }
  List<api.ViewRef> createView(api.ProtoViewRef protoViewRef) {
    return _collectComponentChildViewRefs(
        this._viewFactory.getView(_resolveProtoView(protoViewRef)));
  }
  destroyView(api.ViewRef viewRef) {
    this._viewFactory.returnView(_resolveView(viewRef));
  }
  void insertViewIntoContainer(api.ViewContainerRef vcRef, api.ViewRef viewRef,
      [atIndex = -1]) {
    _resolveViewContainer(vcRef).insert(_resolveView(viewRef), atIndex);
  }
  void detachViewFromContainer(api.ViewContainerRef vcRef, num atIndex) {
    _resolveViewContainer(vcRef).detach(atIndex);
  }
  void setElementProperty(api.ViewRef viewRef, num elementIndex,
      String propertyName, dynamic propertyValue) {
    _resolveView(viewRef).setElementProperty(
        elementIndex, propertyName, propertyValue);
  }
  void setDynamicComponentView(
      api.ViewRef viewRef, num elementIndex, api.ViewRef nestedViewRef) {
    _resolveView(viewRef).setComponentView(
        this._shadowDomStrategy, elementIndex, _resolveView(nestedViewRef));
  }
  void setText(api.ViewRef viewRef, num textNodeIndex, String text) {
    _resolveView(viewRef).setText(textNodeIndex, text);
  }
  void setEventDispatcher(api.ViewRef viewRef, dynamic dispatcher) {
    _resolveView(viewRef).setEventDispatcher(dispatcher);
  }
}
