library angular2.src.render.dom.view.view_container;

import "package:angular2/src/facade/lang.dart"
    show isPresent, isBlank, BaseException;
import "package:angular2/src/facade/collection.dart"
    show ListWrapper, MapWrapper, List;
import "package:angular2/src/dom/dom_adapter.dart" show DOM;
import "view.dart" as viewModule;
import "../shadow_dom/light_dom.dart" as ldModule;
import "view_factory.dart" as vfModule;

class ViewContainer {
  vfModule.ViewFactory _viewFactory;
  var templateElement;
  List<viewModule.RenderView> _views;
  ldModule.LightDom _lightDom;
  ldModule.LightDom _hostLightDom;
  bool _hydrated;
  ViewContainer(vfModule.ViewFactory viewFactory, templateElement) {
    this._viewFactory = viewFactory;
    this.templateElement =
        templateElement; // The order in this list matches the DOM order.
    this._views = [];
    this._hostLightDom = null;
    this._hydrated = false;
  }
  hydrate(ldModule.LightDom destLightDom, ldModule.LightDom hostLightDom) {
    this._hydrated = true;
    this._hostLightDom = hostLightDom;
    this._lightDom = destLightDom;
  }
  dehydrate() {
    if (isBlank(this._lightDom)) {
      for (var i = this._views.length - 1; i >= 0; i--) {
        var view = this._views[i];
        ViewContainer.removeViewNodes(view);
        this._viewFactory.returnView(view);
      }
      this._views = [];
    } else {
      for (var i = 0; i < this._views.length; i++) {
        var view = this._views[i];
        this._viewFactory.returnView(view);
      }
      this._views = [];
      this._lightDom.redistribute();
    }
    this._hostLightDom = null;
    this._lightDom = null;
    this._hydrated = false;
  }
  viewModule.RenderView get(num index) {
    return this._views[index];
  }
  size() {
    return this._views.length;
  }
  _siblingToInsertAfter(num index) {
    if (index == 0) return this.templateElement;
    return ListWrapper.last(this._views[index - 1].rootNodes);
  }
  _checkHydrated() {
    if (!this._hydrated) throw new BaseException(
        "Cannot change dehydrated ViewContainer");
  }
  viewModule.RenderView insert(view, [atIndex = -1]) {
    if (!view.hydrated()) {
      view.hydrate(this._hostLightDom);
    }
    if (atIndex == -1) atIndex = this._views.length;
    ListWrapper.insert(this._views, atIndex, view);
    if (isBlank(this._lightDom)) {
      ViewContainer.moveViewNodesAfterSibling(
          this._siblingToInsertAfter(atIndex), view);
    } else {
      this._lightDom.redistribute();
    } // new content tags might have appeared, we need to redistribute.
    if (isPresent(this._hostLightDom)) {
      this._hostLightDom.redistribute();
    }
    return view;
  } /**
   * The method can be used together with insert to implement a view move, i.e.
   * moving the dom nodes while the directives in the view stay intact.
   */
  detach(num atIndex) {
    this._checkHydrated();
    var detachedView = this.get(atIndex);
    ListWrapper.removeAt(this._views, atIndex);
    if (isBlank(this._lightDom)) {
      ViewContainer.removeViewNodes(detachedView);
    } else {
      this._lightDom.redistribute();
    } // content tags might have disappeared we need to do redistribution.
    if (isPresent(this._hostLightDom)) {
      this._hostLightDom.redistribute();
    }
    return detachedView;
  }
  contentTagContainers() {
    return this._views;
  }
  List nodes() {
    var r = [];
    for (var i = 0; i < this._views.length; ++i) {
      r = ListWrapper.concat(r, this._views[i].rootNodes);
    }
    return r;
  }
  static moveViewNodesAfterSibling(sibling, view) {
    for (var i = view.rootNodes.length - 1; i >= 0; --i) {
      DOM.insertAfter(sibling, view.rootNodes[i]);
    }
  }
  static removeViewNodes(view) {
    var len = view.rootNodes.length;
    if (len == 0) return;
    var parent = view.rootNodes[0].parentNode;
    for (var i = len - 1; i >= 0; --i) {
      DOM.removeChild(parent, view.rootNodes[i]);
    }
  }
}
