library angular2.test.core.compiler.view_factory_spec;

import "package:angular2/test_lib.dart"
    show describe, ddescribe, it, iit, xit, xdescribe, expect, beforeEach, el;
import "package:angular2/src/core/compiler/view_factory.dart" show ViewFactory;
import "package:angular2/src/core/compiler/view.dart"
    show AppProtoView, AppView;
import "package:angular2/change_detection.dart" show dynamicChangeDetection;

main() {
  ViewFactory createViewFactory({capacity}) {
    return new ViewFactory(capacity);
  }
  createPv() {
    return new AppProtoView(null, null,
        dynamicChangeDetection.createProtoChangeDetector("dummy", null));
  }
  describe("RenderViewFactory", () {
    it("should create views", () {
      var pv = createPv();
      var vf = createViewFactory(capacity: 1);
      expect(vf.getView(pv) is AppView).toBe(true);
    });
    describe("caching", () {
      it("should support multiple AppProtoViews", () {
        var pv1 = createPv();
        var pv2 = createPv();
        var vf = createViewFactory(capacity: 2);
        var view1 = vf.getView(pv1);
        var view2 = vf.getView(pv2);
        vf.returnView(view1);
        vf.returnView(view2);
        expect(vf.getView(pv1)).toBe(view1);
        expect(vf.getView(pv2)).toBe(view2);
      });
      it("should reuse the newest view that has been returned", () {
        var pv = createPv();
        var vf = createViewFactory(capacity: 2);
        var view1 = vf.getView(pv);
        var view2 = vf.getView(pv);
        vf.returnView(view1);
        vf.returnView(view2);
        expect(vf.getView(pv)).toBe(view2);
      });
      it("should not add views when the capacity has been reached", () {
        var pv = createPv();
        var vf = createViewFactory(capacity: 2);
        var view1 = vf.getView(pv);
        var view2 = vf.getView(pv);
        var view3 = vf.getView(pv);
        vf.returnView(view1);
        vf.returnView(view2);
        vf.returnView(view3);
        expect(vf.getView(pv)).toBe(view2);
        expect(vf.getView(pv)).toBe(view1);
      });
    });
  });
}
