library angular2.test.render.dom.view.view_spec;

import "package:angular2/test_lib.dart"
    show describe, ddescribe, it, iit, xit, xdescribe, expect, beforeEach, el;
import "package:angular2/src/facade/collection.dart" show ListWrapper;
import "package:angular2/src/render/dom/view/proto_view.dart"
    show RenderProtoView;
import "package:angular2/src/render/dom/view/view.dart" show RenderView;
import "package:angular2/src/render/dom/shadow_dom/shadow_dom_strategy.dart"
    show ShadowDomStrategy;
import "package:angular2/src/render/dom/shadow_dom/light_dom.dart"
    show LightDom;

main() {
  createView() {
    var proto = new RenderProtoView(
        element: el("<div></div>"), isRootView: false, elementBinders: []);
    var rootNodes = [el("<div></div>")];
    var boundTextNodes = [];
    var boundElements = [el("<div></div>")];
    var viewContainers = [];
    var contentTags = [];
    var eventManager = null;
    return new RenderView(proto, rootNodes, boundTextNodes, boundElements,
        viewContainers, contentTags, eventManager);
  }
  createShadowDomStrategy(log) {
    return new FakeShadowDomStrategy(log);
  }
  describe("RenderView", () {
    var log, strategy;
    beforeEach(() {
      log = [];
      strategy = createShadowDomStrategy(log);
    });
    describe("setComponentView", () {
      it("should redistribute when a component is added to a hydrated view",
          () {
        var hostView = createView();
        var childView = createView();
        hostView.hydrate(null);
        hostView.setComponentView(strategy, 0, childView);
        expect(log[0]).toEqual(["redistribute"]);
      });
      it("should not redistribute when a component is added to a dehydrated view",
          () {
        var hostView = createView();
        var childView = createView();
        hostView.setComponentView(strategy, 0, childView);
        expect(log).toEqual([]);
      });
    });
  });
}
class FakeShadowDomStrategy extends ShadowDomStrategy {
  var log;
  FakeShadowDomStrategy(log) : super() {
    /* super call moved to initializer */;
    this.log = log;
  }
  LightDom constructLightDom(
      RenderView lightDomView, RenderView shadowDomView, element) {
    return new FakeLightDom(this.log, lightDomView, shadowDomView, element);
  }
}
class FakeLightDom extends LightDom {
  var log;
  FakeLightDom(log, RenderView lightDomView, RenderView shadowDomView, element)
      : super(lightDomView, shadowDomView, element) {
    /* super call moved to initializer */;
    this.log = log;
  }
  redistribute() {
    ListWrapper.push(this.log, ["redistribute"]);
  }
}
